//+------------------------------------------------------------------+
//| 10:06 AM 8/8/2015                                     arrays.mqh |
//|                              Copyright  2015, William H. Roeder |
//|                                        mailto:WHRoeder@yahoo_com |
//+------------------------------------------------------------------+
/** @file
 * @brief Provides standalone template functions that deal with arrays. Both
 * vectors (insert at end) and deques (insert at beginning) are supported, as
 * are single and multi-dimensional arrays.                                   */
//+------------------------------------------------------------------+
#ifndef ARRAYS_MQH
#define ARRAYS_MQH
#include "basic_types.mqh"                // INDEX
#include "relation_operations.mqh"        // Less, Like
//+------------------------------------------------------------------+
/** This structure just contains static functions for name space control.
 * Functions that can resize an array take an (optional) string. This is used
 * report the caller's identification (via `assert_failure`) if the
 * `ArrayResize` fails.                                                       */
struct   Arrays{
/// Returns the row count (the first or only dimension) of the _array_.
template <typename Datatype>
static COUNT   get_size(const Datatype& arr[]){ // ArrayRange is used instead
   return ArrayRange(arr,0);                    // of ArraySize, to allow array
}                                               // to be multi-dimensional.
/** Insures the _vector_ has at least _nRows_ by calling resize_vector() when
 * necessary. @returns False if the _vector_ couldn't be resized.             */
template <typename Datatype>
static bool    set_capacity(Datatype&  vector[],      ///<[in,out] The vector.
                            COUNT      nRows,         ///<[in]Row count.
                            string     msg="")        /**<[in]Diagnostic */{
   if(msg == "")                       msg = __FUNCTION__;
   return get_size(vector) >= nRows || resize_vector(vector, nRows, msg);
}
/** Adds or remove elements from the end of the _vector_. Added elements are \b
 * not initialized. If `ArrayResize()` fails assert_failure() is called using
 * _msg_. @returns False if the _vector_ couldn't be resized.                 */
template <typename Datatype>
static bool    resize_vector(Datatype& vector[],      ///<[in,out] The vector.
                             COUNT     nRows,         ///<[in]Row count.
                             string    msg="")        /**<[in]Diagnostic. */{
   COUNT    current  = get_size(vector);
   COUNT    extra    = MathMax(nRows - current, 16);
   if(ArrayResize(vector, nRows, extra) >= 0)                     return true;
   if(msg == "")  msg = __FUNCTION__;
   assert_failure(StringFormat("%s: ArrayResize(%i->%i) Failed: %i", msg,
                               current, nRows, _LastError) );     return false;
}
/** Adds or remove elements from the end of an _vector_. Added elements \b are
 * initialized to the given _value_.
 * @returns False if the _vector_ couldn't be resized.                        */
template <typename Datatype>
static bool    enlarge_vector(const Datatype&value,   ///<[in]New value.
                              Datatype&      vector[],///<[in,out] The vector.
                              COUNT          nRows,   ///<[in]Row count.
                              string         msg="")  /**<[in]Diagnostic. */{
   if(msg == "")                             msg = __FUNCTION__;
   INDEX    iEmpty   = get_size(vector);
   if(!resize_vector(vector, nRows, msg)) return false;
   while(nRows > iEmpty)   vector[--nRows] = value;
   return true;
}
/// Adds the given _value_ to the end of the vector.
template <typename Datatype>
static bool    push_back(const Datatype&  value,   ///<[in]New element value.
                         Datatype&        vector[],///<[in,out] The vector.
                         string           msg="")  /**<[in]Diagnostic. */{
   if(msg == "")                          msg = __FUNCTION__;
   return enlarge_vector(value, vector, get_size(vector) + 1, msg);
}
/** Removes one element from the end of the _vector_.
 * @returns False if the _vector_ couldn't be resized.                        */
template <typename Datatype>
static bool    pop_back(Datatype&   vector[],         ///<[in,out] The vector.
                        string      msg="")           /**<[in]Diagnostic. */{
   if(msg == "")                    msg = __FUNCTION__;
   return resize_vector(vector, get_size(vector) - 1, msg);
}
/** Finds the given _value_ in the _vector_, removes it via erase_at()
 * @returns False if the _vector_ couldn't be resized.                        */
template <typename Datatype>
static bool    erase_value(const Datatype& value,  ///<[in]New element value.
                           Datatype&     vector[], ///<[in,out] The vector.
                           string         msg="")  /**<[in]Diagnostic. */{
   if(msg == "")                          msg = __FUNCTION__;
   INDEX    iEnd  = ArraySize(vector);
   INDEX    iPos  = find(value, vector, iEnd);
   return (iPos != iEnd) ? erase_at(vector, iPos, msg) : true;
}
/** Removes the element at the given _index_ by overwriting it with the last
 * element and remove that via resize_vector().
 * @returns False if the _vector_ couldn't be resized.                        */
template <typename Datatype>
static bool    erase_at(Datatype&   vector[],         ///<[in,out] The vector.
                        INDEX       index,            ///<[in]Location.
                         string     msg="")           /**<[in]Diagnostic. */{
   if(msg == "")                    msg = __FUNCTION__;
   INDEX    iLast = get_size(vector) - 1;
   vector[index] = vector[iLast];
   return resize_vector(vector, iLast, msg);
}
/** Removes the element at the given _index_ by by copying all higher elements
 * down one position and removes the last element via resize_vector().
 * @returns False if the _vector_ couldn't be resized.                        */
template <typename Datatype>
static bool    erase_ordered(Datatype& vector[],      ///<[in,out] The vector.
                             INDEX     index,         ///<[in]Location.
                             string    msg="")        /**<[in]Diagnostic. */{
   if(msg == "")                       msg = __FUNCTION__;
   for(INDEX iLast = get_size(vector) - 1; index < iLast; ++index)
      vector[index] = vector[index + 1];
   return resize_vector(vector, iLast, msg);
}
/** Sets the _vector_ to zero size via resize_vector().
 * @returns False if the _array_ couldn't be resized.                         */
template <typename Datatype>
static bool    clear_array(Datatype&   array[],       ///<[in,out] The array.
                         string        msg="")        /**<[in]Diagnostic. */{
   if(msg == "")                       msg = __FUNCTION__;
   return resize_vector(array, 0, msg);
}
/** Adds or remove elements from the beginning of a _deque_. Elements above
 * added elements are not moved due to the use of `ArraySetAsSeries`.
 * Therefor `O(resize_buffer) == O(resize_vector)`. Added elements are _not_
 * initialized. @returns False if the _deque_ couldn't be resized.            */
template <typename Datatype>
static bool    resize_buffer(Datatype& deque[],       ///<[in,out] The deque.
                             COUNT     nRows,         ///<[in]Row count.
                             string    msg="")        /**<[in]Diagnostic. */{
   if(msg == "")                       msg = __FUNCTION__;
   // This enlarges the arr and shifts values B[2]=B[1]; B[1]=B[0]; B[0]=prob 0.
   bool     isSeries = ArrayGetAsSeries(deque);
   ArraySetAsSeries(deque, !isSeries);
   if(!resize_vector(deque, nRows, msg) )       return false;
   ArraySetAsSeries(deque, isSeries);           return true;
}
/** Adds elements from the beginning of the _deque_. Added elements \b are
 * initialized to the given _value_.
 * @returns False if the _deque_ couldn't be resized.                         */
template <typename Datatype>
static bool    enlarge_buffer(const Datatype&value,   ///<[in]New value.
                              Datatype&      deque[], ///<[in,out] The deque.
                              COUNT          nRows,   ///<[in]Row count.
                              string         msg="")  /**<[in]Diagnostic. */{
   if(msg == "")                             msg = __FUNCTION__;
   INDEX    iNew = get_size(deque);
   if(!resize_buffer(deque, nRows, msg))     return false;
   for(; iNew < nRows; ++iNew)   deque[iNew] = value;
   return true;
}
/** Adds _value_ to the beginning of the _deque_ using resize_buffer().
 * @returns False if the _deque_ couldn't be resized.                         */
template <typename Datatype>
static bool    push_front(const Datatype& value,   ///<[in]New element value.
                          Datatype&       deque[], ///<[in,out] The deque.
                          string          msg="")  /**<[in]Diagnostic. */{
   if(msg == "")                          msg = __FUNCTION__;
   COUNT    nRows = get_size(deque);
   return enlarge_buffer(value, deque, nRows + 1, msg);
}
/** Remove element zero (the beginning) of a _deque_ using resize_buffer().
 * @returns False if the _deque_ couldn't be resized.                         */
template <typename Datatype>
static bool    pop_front(Datatype&  deque[],          ///<[in,out] The deque.
                         string     msg="")           /**<[in]Diagnostic. */{
   if(msg == "")                    msg = __FUNCTION__;
   COUNT    nRows = get_size(deque);
   return resize_buffer(deque, nRows - 1, msg);
}
/// Copy an array.
template <typename Datatype>
static bool    copy(Datatype&       target[],   ///<[out] Output array.
                    const Datatype& source[],   ///<[in]Input array
                    INDEX        iEnd=MAX_INDEX,///<[in]Input one past last.
                    INDEX           iBeg=0,     ///<[in]Input starting index.
                    INDEX           iDst=0,     ///<[in]Output starting index.
                    string          msg="")     /**<[in]Diagnostic. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(source);
   COUNT    count = iEnd - iBeg;
   COUNT    result   = ArrayCopy(target, source, iDst, iBeg, count);
   if(count == result + iDst) return true;
   if(msg == "")  msg = __FUNCTION__;
   assert_failure(StringFormat("%s ac(%i)=%i %i", msg, count + iDst, result,
                               _LastError) ); 
   return false;
}
/** Linear search an _array_ for a specific _value_ using Like. @returns INDEX
 * of the first element found with _value_ or _end_ if not found.             */
template <typename Datatype>
static INDEX   find(const Datatype& value,         ///<[in]Value searching for.
                    const Datatype& arr[],         ///<[in]Array to search.
                    INDEX           iEnd=MAX_INDEX,///<[in]One past last.
                    INDEX           iBeg=0,        ///<[in]Starting index.
                    bool            isAscend=true) /**<[in]Direction of the
                     search. In case there are duplicate values in the array.*/{
   Like  comp;    return find(value, comp, arr, iEnd, iBeg, isAscend);
}
/** Linear search an _array_ for a specific _value_, i.e.\ where the supplied
 * _comp_ `class` or `struct` method `is_same(Datatype, Datatype)` returns true.
 * @returns Index of the first element with _value_ or _end_ if not found.    */
template <typename Type1, typename Type2, typename BinaryPredicate>
static INDEX   find(const Type1&          value,///< [in]Value searching for.
                    const BinaryPredicate& comp,///< [in]Comp. class/struct.
                    const Type2&          arr[],///< [in]Array to search.
                    INDEX        iEnd=MAX_INDEX,///<[in]One past last.
                    INDEX                iBeg=0,///< [in]Starting index.
                    bool          isAscend=true)/**< [in]Direction of the search
                           in case there are duplicate values in the array.*/{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(isAscend){
      for(; iBeg < iEnd; ++iBeg) if(comp.is_same(arr[iBeg], value) ) break;
      return iBeg;
   }
   for(INDEX iPos = iEnd; iPos > iBeg;)
                              if(comp.is_same(arr[--iPos], value) )  break;
   return iEnd;
}
/** Returns the index in a __sorted__ _array_, the first element greater than
 * _value_ using Less.                                                        */
template <typename Type1, typename Type2>
static INDEX   upper_bound(const Type1&   value,   ///<[in]Value searching for.
                           const Type2&   arr[],   ///<[in]Array to search.
                           INDEX    iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX          iBeg=0)  /**<[in]Starting index.*/{
   Less  comp;    return upper_bound(value, comp, arr, iEnd, iBeg);
}
/** Returns the index in a __sorted__ _array_, the first element which compares
 * larger than _value_ using the supplied _comp_ `class` or `struct` method
 * `is_before(Datatype, Datatype)` which creates a [strict weak ordering](
 * http://simple.wikipedia.org/wiki/Strict_weak_ordering "Strict weak ordering -
 * Simple English Wikipedia, the free encyclopedia"). If no value is larger,
 * returns _end_.                                                             */
template <typename Type1, typename Type2, typename BinaryPredicate>
static INDEX   upper_bound(const Type1&   value,   ///<[in]Value searching for.
                       const BinaryPredicate& comp,///<[in]Comp. class/struct.
                           const Type2&   arr[],   ///<[in]Array to search.
                           INDEX    iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX            iBeg=0)/**<[in]Starting index. */{
   if(iEnd == _INDEX)   iEnd  = get_size(arr);
   while(iEnd > iBeg){
      INDEX    iMid = iBeg + (iEnd - iBeg) / 2;
      if(comp.is_before(value, arr[iMid]) )  iEnd  = iMid;
      else                                   iBeg  = iMid + 1;
   }
   return iBeg;
}
/** Returns the index in a __sorted__ _array_, the first element greater than or
 * equal to _value_ using Less. If all values are less than _value_, returns
 * _end_.                                                                     */
template <typename Type1, typename Type2>
static INDEX   lower_bound(const Type1&   value,   ///<[in]Value searching for.
                           const Type2&   arr[],   ///<[in]Array to search.
                           INDEX    iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX          iBeg=0)  /**<[in]Starting index. */{
   Less  comp;    return lower_bound(arr, iBeg, iEnd, value, comp);
}
/** Returns the index in a __sorted__ _array_, the first element which does not
 * compare smaller than _value_ using the supplied _comp_ `class` or `struct`
 * method `is_before(Datatype, Datatype)` which creates a [strict weak
 * ordering](http://simple.wikipedia.org/wiki/Strict_weak_ordering "Strict weak
 * ordering - Simple English Wikipedia, the free encyclopedia"). If no value is
 * larger, returns _end_.                                                     */
template <typename Type1, typename Type2, typename BinaryPredicate>
static INDEX   lower_bound(const Type1&   value,   ///<[in]Value searching for.
                       const BinaryPredicate& comp,///<[in]Comp. class/struct.
                           const Type2&   arr[],   ///<[in]Array to search.
                           INDEX    iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX          iBeg=0)  /**<[in]Starting index. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   while(iBeg < iEnd){
      INDEX    iMid = iBeg   + (iEnd - iBeg) / 2;
      if(comp.is_before(arr[iMid], value) )  iBeg  = iMid + 1;
      else                                   iEnd  = iMid;
  }
  return iBeg;
}
/** Finds the INDEX of the maximum element in an _array_ using Less. This is
 * equivalent to `ArrayMaximum()` on numerical arrays, except:
 * - In the case of duplicates, there is control of which index to return.
 * - Extends to non-numerical datatypes where `operator<()` is defined.       */
template <typename Datatype>
static INDEX   max_element(const Datatype&   arr[],   ///<[in]Array to search.
                           INDEX       iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX             iBeg=0,  ///<[in]Starting index.
                           bool         isAscend=true)/**<[in]Direction of the
                   * search in case there are duplicate values in the array. */{
   Less  comp;    return max_element(comp, arr, iEnd, iBeg, isAscend);
}
/** Finds the largest element in an _array_, where largest means where the
 * supplied _comp_ `class` or `struct` method `is_before(Datatype, Datatype)`
 * creates a [strict weak ordering](
 * http://simple.wikipedia.org/wiki/Strict_weak_ordering "Strict weak ordering -
 * Simple English Wikipedia, the free encyclopedia").                         */
template <typename Datatype, typename BinaryPredicate>
static INDEX   max_element(
                     const BinaryPredicate&  comp, ///<[in]Comp. class/struct.
                     const Datatype&         arr[],///<[in]Array to search.
                     INDEX          iEnd=MAX_INDEX,///<[in]One past last.
                     INDEX                  iBeg=0,///<[in]Starting index.
                     bool            isAscend=true)/**<[in]Direction of the
                   * search in case there are duplicate values in the array. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(iEnd == iBeg)  return iEnd;
   if(isAscend){
      INDEX    iExtreme = iBeg++;   Datatype extreme  = arr[iExtreme], value;
      for(; iBeg < iEnd; ++iBeg){
         value = arr[iBeg];
         if(comp.is_before(extreme, value) ){iExtreme = iBeg;  extreme = value;}
      }
      return iExtreme;
   }
   INDEX    iExtreme = --iEnd;   Datatype extreme  = arr[iExtreme], value;
   while(iEnd > iBeg){
      value = arr[--iEnd];
      if(comp.is_before(extreme, value) ){   iExtreme = iEnd;  extreme = value;}
   }
   return iExtreme;
}
/** Finds the INDEX of the minimum element in an _array_ using Less. This
 * equivalent to `ArrayMinimum()` on numerical arrays, except:
 * - In the case of duplicates, there is control of which index to return.
 * - Extends to non-numerical datatypes where `operator<()` is defined.       */
template <typename Datatype>
static INDEX   min_element(const Datatype&   arr[],   ///<[in]Array to search.
                           INDEX       iEnd=MAX_INDEX,///<[in]One past last.
                           INDEX             iBeg=0,  ///<[in]Starting index.
                           bool         isAscend=true)/**<[in]Direction of the
                   * search in case there are duplicate values in the array. */{
   Less  comp;    return min_element(comp, arr, iEnd, iBeg, isAscend);
}
/** Finds the smallest element in an _array_, where smallest means where the
 * supplied _comp_ `class` or `struct` method `is_before(Datatype, Datatype)`
 * creates a [strict weak ordering](
 * http://simple.wikipedia.org/wiki/Strict_weak_ordering "Strict weak ordering -
 * Simple English Wikipedia, the free encyclopedia").                         */
template <typename Datatype, typename BinaryPredicate>
static INDEX   min_element(
                     const BinaryPredicate&  comp, ///<[in]Comp. class/struct.
                     const Datatype&         arr[],///<[in]Array to search.
                     INDEX          iEnd=MAX_INDEX,///<[in]One past last.
                     INDEX                  iBeg=0,///<[in]Starting index.
                     bool            isAscend=true)/**<[in]Direction of the
                   * search in case there are duplicate values in the array. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(iEnd == iBeg)  return iEnd;
   if(isAscend){
      INDEX    iExtreme = iBeg++;   Datatype extreme  = arr[iExtreme], value;
      for(; iBeg < iEnd; ++iBeg){
         value = arr[iBeg];
         if(comp.is_before(value, extreme) ){iExtreme = iBeg;  extreme = value;}
      }
      return iExtreme;
   }
   INDEX    iExtreme = --iEnd;   Datatype extreme  = arr[iExtreme], value;
   while(iEnd > iBeg){
      value = arr[--iEnd];
      if(comp.is_before(value, extreme) ){   iExtreme = iEnd;  extreme = value;}
   }
   return iExtreme;
}
/** Finds the INDEX of the maximum local extream in an _array_ using Less,
 * i.e.\. moving up hill until you reach the top.                             */
template <typename Datatype>
static INDEX   max_extreama(const Datatype&  arr[],   ///<[in]Array to search.
                            COUNT            size,    ///<[in]Size of a local.
                            INDEX      iEnd=MAX_INDEX,///<[in]One past last.
                            INDEX            iBeg=0,  ///<[in]Starting index.
                            bool        isAscend=true)/**<[in]Direction of the
                                                       * search */{
   Less  comp;    return max_extreama(comp, arr, size, iEnd, iBeg, isAscend);
}
/** Finds the INDEX of the maximum local extream in an _array_ using Less,
 * i.e.\. moving up hill until you reach the top. @see max_element.           */
template <typename Datatype, typename BinaryPredicate>
static INDEX   max_extreama(
                     const BinaryPredicate&  comp, ///<[in]Comp. class/struct.
                     const Datatype&         arr[],///<[in]Array to search.
                     COUNT                   size, ///<[in]Size of a local.
                     INDEX          iEnd=MAX_INDEX,///<[in]One past last.
                     INDEX                  iBeg=0,///<[in]Starting index.
                     bool            isAscend=true)/**<[in]Direction of the
                                                    * search. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(iEnd == iBeg)  return iEnd;
   if(isAscend){
      INDEX    iExtreme = iBeg++;   Datatype extreme  = arr[iExtreme], value;
      INDEX    iLimit   = MathMin(iEnd, iExtreme + size);
      for(; iBeg < iLimit; ++iBeg){
         value = arr[iBeg];
         if(comp.is_before(extreme, value) ){
            iExtreme = iBeg;  extreme = value;
            iLimit   = MathMin(iEnd, iExtreme + size);
      }  }
      return iExtreme;
   }
   INDEX    iExtreme = --iEnd;   Datatype extreme = arr[iExtreme], value;
   // INDEX is uint, can't use MathMax(iBeg, iEnd - size) as that might go
   // negitive. Instead (iEnd - size < iBeg) == (iEnd < iBeg + size)
   --size;  // iEnd decremented first so size is one smaller.
   INDEX    iLast    = (iExtreme <= iBeg + size) ? iBeg : iExtreme - size;
   while(iEnd > iLast){
      value = arr[--iEnd];
      if(comp.is_before(extreme, value) ){
         iExtreme = iEnd;  extreme = value;
         iLast    = (iExtreme <= iBeg + size) ? iBeg : iExtreme - size;
   }  }
   return iExtreme;
}
/** Finds the INDEX of the minimum local extream in an _array_ using Less,
 * i.e.\. moving down hill until you reach the bottom.                        */
template <typename Datatype>
static INDEX   min_extreama(const Datatype&  arr[],   ///<[in]Array to search.
                            COUNT            size,    ///<[in]Size of a local.
                            INDEX      iEnd=MAX_INDEX,///<[in]One past last.
                            INDEX            iBeg=0,  ///<[in]Starting index.
                            bool        isAscend=true)/**<[in]Direction of the
                                                       * search */{
   Less  comp;    return min_extreama(comp, arr, size, iEnd, iBeg, isAscend);
}
/** Finds the INDEX of the minimum local extream in an _array_ using Less,
 * i.e.\. moving down hill until you reach the top. @see min_element.         */
template <typename Datatype, typename BinaryPredicate>
static INDEX   min_extreama(
                     const BinaryPredicate&  comp, ///<[in]Comp. class/struct.
                     const Datatype&         arr[],///<[in]Array to search.
                     COUNT                   size, ///<[in]Size of a local.
                     INDEX          iEnd=MAX_INDEX,///<[in]One past last.
                     INDEX                  iBeg=0,///<[in]Starting index.
                     bool            isAscend=true)/**<[in]Direction of the
                                                    * search */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(iEnd == iBeg)  return iEnd;
   if(isAscend){
      INDEX    iExtreme = iBeg++;   Datatype extreme  = arr[iExtreme], value;
      INDEX    iLimit   = MathMin(iEnd, iExtreme + size);
      for(; iBeg < iLimit; ++iBeg){
         value = arr[iBeg];
         if(comp.is_before(value, extreme) ){
            iExtreme = iBeg;  extreme = value;
            iLimit   = MathMin(iEnd, iExtreme + size);
         }
      }  
      return iExtreme;
   }
   INDEX    iExtreme = --iEnd;   Datatype extreme = arr[iExtreme], value;
   // INDEX is uint, can't use MathMax(iBeg, iEnd - size) as that might go
   // negitive. Instead (iEnd - size < iBeg) == (iEnd < iBeg + size)
   --size;  // iEnd decremented first so size is one smaller.
   INDEX    iLast    = (iExtreme <= iBeg + size) ? iBeg : iExtreme - size;
   while(iEnd > iBeg){
      value = arr[--iEnd];
      if(comp.is_before(value, extreme) ){
         iExtreme = iEnd;  extreme = value;
         iLast    = (iExtreme <= iBeg + size) ? iBeg : iExtreme - size;
   }  }
   return iExtreme;
}
/// Performs an insertion sort on the _array_ using Less.
template <typename Datatype>
static void    insertion_sort(Datatype&   arr[],   ///<[in,out] Array to search.
                              INDEX iEnd=MAX_INDEX,///<[in]One past last.
                              INDEX       iBeg=0)  /**<[in]Starting index.*/{
   Less  comp;    return insertion_sort(comp, arr, iEnd, iBeg);
}
/** Perform an insertion sort on the _array_. The supplied _comp_ `class` or
 * `struct` method `is_before(Datatype, Datatype)` creates a [strict weak
 * ordering](http://simple.wikipedia.org/wiki/Strict_weak_ordering "Strict weak
 * ordering - Simple English Wikipedia, the free encyclopedia").              */
template <typename Datatype, typename BinaryPredicate>
static void     insertion_sort(
                     const BinaryPredicate&  comp, ///<[in]Comp. class/struct.
                     Datatype&               arr[],///<[in,out] Array to search.
                     INDEX          iEnd=MAX_INDEX,///<[in]One past last.
                     INDEX                  iBeg=0)/**<[in]Starting index. */{
   if(iEnd == MAX_INDEX)   iEnd  = get_size(arr);
   if(iEnd == iBeg)  return;
   // This is more complicated than a standard insertion sort because array
   // access in MT4 is 10x slower than variable access.
   Datatype placing;
   Datatype previous = arr[iBeg];
   for(INDEX iPlace = iBeg + 1; iPlace < iEnd; ++iPlace){
      placing  = arr[iPlace];                   // Where does this go?
      if(comp.is_before(previous, placing) ){   // Already in correct position.
         previous = placing;  continue;      }  // Prime for next iteration.
      INDEX    iEmpty = iPlace;                 // Value in placing.
      do{
         arr[iEmpty] = previous;                // Move up.
         if(--iEmpty == iBeg) break;            // No below.
         previous = arr[iEmpty - 1];            // Next one.
      } while(comp.is_before(placing, previous) );
      arr[iEmpty] = placing;                    // Insert.
      previous = arr[iPlace];                   // Prime for next iteration.
}  }
 private:
   char onebyte;///< Prevent "struct has no members, size assigned to 1 byte
};
//+- Arrays ---------------------------------------------------------+
#endif   // ARRAYS_MQH
//+------------------------------------------------------------------+
